/**
 * {按钮组件}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/03/06 15:17:25
 */

import {
  nextTick,
  css,
  html,
  Component,
  classMap,
  outsideClick
} from '@bd/core'
import '../icon/index.js'
import '../scroll/index.js'

const ANIMATION = {
  duration: 100,
  custom: [{ transform: 'scaleY(0)' }, { transform: 'scaleY(1)' }]
}
class Input extends Component {
  static props = {
    readOnly: false,
    autofocus: false,
    disabled: false,
    clearable: false,
    icon: '',
    size: 'l',
    placeholder: '',
    maxlength: null,
    minlength: null,
    value: {
      type: String,
      default: '',
      attribute: false
    },
    lazy: 0 // 并发拦截时间, 单位毫秒
  }
  #list = []
  #originList = []
  #isComposing = false
  #selectIndex = -1
  #listShowing = false
  #stamp = 0
  static styles = [
    css`
      :host {
        position: relative;
        display: inline-flex;
        min-width: 228px;
        height: 36px;
        user-select: none;
        -moz-user-select: none;
        color: var(--color-dark-1);
        border-radius: 3px;
        cursor: text;
        transition: box-shadow 0.15s linear;
      }

      .label {
        flex: 1;
        display: flex;
        justify-content: center;
        align-items: center;
        height: 100%;
        font-size: 14px;
        border: 1px solid var(--color-grey-2);
        border-radius: inherit;
        background: var(--bg-color, #fff);
        color: inherit;
        cursor: inherit;

        input {
          flex: 1;
          min-width: 36px;
          width: 0;
          height: 100%;
          padding: 0 8px;
          border: 0;
          border-radius: inherit;
          font: inherit;
          color: inherit;
          background: none;
          outline: none;
          box-shadow: none;
          cursor: inherit;

          &::placeholder {
            color: var(--color-grey-1);
          }
        }

        .prepend,
        .append {
          display: flex;
          justify-content: center;
          align-items: center;
          width: auto;
          white-space: nowrap;
        }

        &[prepend] .prepend,
        &[append] .append {
          display: flex;
        }
        &[prepend] input,
        &[append] input {
          min-width: 64px;
        }
        /* ----- */
        .close {
          --size: 18px;
          margin: 0 8px 0 4px;
          padding: 4px;
          border-radius: 50%;
          color: var(--color-grey-2);
          cursor: pointer;
          transition: background 0.15s linear;

          &:hover {
            background: var(--color-plain-1);
          }
        }
      }

      .suggestion {
        overflow: hidden;
        position: absolute;
        z-index: 1;
        left: 0;
        top: calc(100% + 4px);
        width: 100%;
        height: auto;
        max-height: 200px;
        padding: 4px 0;
        border-radius: 4px;
        // background: var(--color-plain-1);
        box-shadow: 0 2px 5px rgba(0, 0, 0, 0.15);
        transform-origin: top;
        // transition: transform linear 00s;
        // transform: scaleY(0);
        wc-scroll {
          max-height: 200px;
        }
        .list {
          width: 100%;
        }
        &.hide {
          display: none;
        }

        li {
          overflow: hidden;
          width: 100%;
          height: 30px;
          line-height: 30px;
          padding: 0 8px;
          text-overflow: ellipsis;
          white-space: nowrap;
          cursor: pointer;

          &:hover,
          &[focus] {
            background: var(--color-plain-2);
          }
        }
      }

      /* 额外样式 */
      :host([round]) {
        border-radius: 26px;

        .label input {
          padding: 0 10px;
          // margin: 0 18px;
        }

        .label[prepend] input,
        .label[append] input {
          padding: 0 5px;
        }
      }

      :host([size='large']) {
        min-width: 234px;
        .label {
          height: 52px;
          font-size: 18px;

          input {
            padding: 0 16px;
          }
        }
        .prepend,
        .append {
          height: 48px;
          padding: 0 16px;
        }

        .icon {
          --size: 24px;
          margin: 0 20px 0 4px;
        }
      }
      :host([size='medium']) {
        min-width: 160px;
        .label {
          height: 44px;

          input {
            padding: 0 10px;
          }
        }
        .prepend,
        .append {
          height: 40px;
        }
      }
      :host([size='small']) {
        min-width: 96px;
        .label {
          height: 32px;

          input {
            padding: 0 6px;
          }
        }
        .prepend,
        .append {
          height: 28px;
        }
      }
      :host([size='mini']) {
        min-width: 72px;
        .label {
          height: 26px;
          font-size: 12px;
        }
        .icon {
          --size: 14px;
        }
        .prepend,
        .append {
          height: 22px;
        }
      }

      :host(:focus-within) {
        box-shadow: 0 0 0 2px var(--color-plain-a);
      }
      /* --- */
      :host([disabled]) {
        pointer-events: none;
        cursor: not-allowed;

        .label {
          border-color: var(--color-grey-1);
          background: var(--color-plain-1);
          opacity: 0.6;
        }
      }
      :host([readonly]) {
        cursor: default;
      }

      :host([no-border]),
      :host(:focus-within[no-border]) {
        .label {
          border: 0;

          &[prepend] input {
            padding-left: 2px;
          }
          &[append] input {
            padding-right: 2px;
          }
        }
        .prepend,
        .append {
          border: 0;
        }
      }
    `,
    //尺寸
    css`
      @use 'sass:map';
      $sizes: (
        m: (
          w: 82px,
          h: 24px,
          f: 12px
        ),
        l: (
          w: 108px,
          h: 32px,
          f: 14px
        ),
        xl: (
          w: 132px,
          h: 36px,
          f: 14px
        ),
        xxl: (
          w: 160px,
          h: 44px,
          f: 14px
        ),
        xxxl: (
          w: 192px,
          h: 52px,
          f: 16px
        )
      );

      @loop $s, $v in $sizes {
        :host([size='#{$s}']) {
          min-width: map.get($v, 'w');
          height: map.get($v, 'h');
          font-size: map.get($v, 'f');
          .label {
            height: map.get($v, 'h');
            font-size: map.get($v, 'f');
          }
          .icon {
            --size: #{map.get($v, 'f')};
          }
        }
        :host([size='#{$s}'][circle]) {
          width: map.get($v, 'h');
          height: map.get($v, 'h');
        }
      }
    `,

    /* ----- 类型(颜色) ----- */
    css`
      $colors: (
        primary: 'teal',
        info: 'blue',
        success: 'green',
        warning: 'orange',
        danger: 'red',
        secondary: 'dark',
        help: 'grey'
      );

      @loop $t, $c in $colors {
        :host([type='#{$t}']:focus-within) {
          box-shadow: 0 0 0 2px var(--color-#{$c}-a);
        }

        :host([type='#{$t}']) .label {
          border-color: var(--color-#{$c}-2);
          .prepend,
          .append {
            border-color: var(--color-#{$c}-a);
          }
        }
      }
    `
  ]
  renderClear() {
    return html`<wc-icon class="close" name="close" @click=${this.clear} />`
  }

  render() {
    let classes = classMap({
      suggestion: true,
      hide: !this.#list.length
    })

    return html`
      <div class="label">
        <slot class="prepend" name="prepend"></slot>
        <input
          ref="input"
          @input=${this.handleInput}
          @change=${this.handleChange}
          @compositionstart=${this.handleCompositionstart}
          @compositionend=${this.handleCompositionend}
          @keydown=${this.handleKeyDown}
          @focus=${this.handleFocus}
          placeholder=${this.placeholder}
          maxlength=${this.maxlength}
          minlength=${this.minlength}
          disabled=${this.disabled}
          readonly=${this.readOnly}
          autofocus=${this.autofocus}
          :value=${this.value}
        />
        ${this.clearable && this.value ? this.renderClear() : ''}
        ${this.icon
          ? html`<wc-icon class="icon" name=${this.icon} />`
          : html`<slot class="append" name="append" />`}
        <div class=${classes} ref="suggestion" #animation=${ANIMATION}>
          <wc-scroll ref="scroller">
            <ul class="list" @click=${this.handleClickItem} ref="list">
              ${this.#list.map(
                (li, idx) =>
                  html`<li
                    focus=${this.#selectIndex === idx ? true : null}
                    index=${idx}
                  >
                    ${li.value}
                  </li>`
              )}
            </ul>
          </wc-scroll>
        </div>
      </div>
    `
  }
  handleCompositionstart() {
    this.#isComposing = true
  }
  handleCompositionend() {
    this.#isComposing = false
    this.filterSuggestList()
  }
  filterSuggestList() {
    if (!this.#originList.length) {
      return
    }
    if (!this.value) {
      this.#list = this.#originList
    } else {
      this.#list = this.#originList.filter(li =>
        li.value.startsWith(this.value)
      )
    }
    if (!this.#listShowing) {
      this.#listShowing = true
      this.$refs.suggestion.$animate()
    }
    this.$requestUpdate()
  }
  handleInput(e) {
    let { lazy } = this
    this.value = e.currentTarget.value
    if (lazy && Date.now() - this.#stamp < lazy) {
      return
    }
    this.#stamp = Date.now()
    if (!this.#isComposing) {
      this.filterSuggestList()
      this.$emit('fetch-suggest', {
        value: this.value,
        send: list => {
          this.#originList = this.#list = list
          this.$requestUpdate()
        }
      })
    }
  }
  handleClickItem(e) {
    let index = e.target.getAttribute('index')
    this.value = this.#list[index].value
    this.#list = [this.#list[index]]
    this.$refs.suggestion.$animate(true)
    this.#listShowing = false
    this.$emit('select')
  }
  clear() {
    this.$refs.input.value = ''
    this.value = ''
    if (this.#originList.length) {
      this.filterSuggestList()
    }
  }
  handleChange() {
    this.$emit('change')
  }
  handleKeyDown(e) {
    let { lazy, minlength, value } = this
    if (e.keyCode === 13) {
      e.preventDefault()
      if (this.#selectIndex > -1 && this.#listShowing) {
        this.value = this.#list[this.#selectIndex].value
        this.#list = [this.#list[this.#selectIndex]]
        this.#selectIndex = 0
        this.$requestUpdate()
        this.$refs.suggestion.$animate(true)
        this.#listShowing = false
        return this.$emit('select') //输入建议存在,则第1次回车的时候, 不触发提交
      }
      if (lazy && Date.now() - this.#stamp < lazy) {
        return
      }
      this.#stamp = Date.now()
      if (minlength && value.length < minlength) {
        return
      }
      return this.$emit('submit')
    }
    if (e.keyCode === 38 || e.keyCode === 40) {
      e.preventDefault()
      let step = e.keyCode === 38 ? -1 : 1

      this.#selectIndex += step
      if (this.#selectIndex < 0) {
        this.#selectIndex = 0
      }
      if (this.#selectIndex > this.#list.length - 1) {
        this.#selectIndex = this.#list.length - 1
      }
      let target = this.$refs.list.children[this.#selectIndex]
      this.$refs.scroller.scrollTop = target.offsetTop - 150
      this.$requestUpdate()
    }
  }
  handleFocus() {
    if (!this.#listShowing) {
      this.#listShowing = true
      this.$refs.suggestion.$animate()
    }
  }
  mounted() {
    if (this.autofocus) {
      // 火狐浏览器需要手动focus()才能聚焦成功
      nextTick(_ => this.$refs.input.focus())
    }
    outsideClick(this, () => {
      this.#listShowing = false
      this.$refs.suggestion.$animate(true)
    })
  }
}

Input.reg('input')
