/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/04/18 09:38:01
 */

import { css, html, Component, bind, styleMap, classMap } from '@bd/core'
import '../icon/index.js'

class Steps extends Component {
  static props = {
    active: {
      type: Number,
      default: 1,
      observer() {
        this.$updateStepStatus()
      }
    },
    vertical: false
  }

  static styles = [
    css`
      :host {
        display: flex;
        justify-content: space-between;
      }

      :host([vertical]) {
        flex-direction: column;
      }
    `
  ]

  $updateStepStatus() {
    ;[...this.children].forEach((it, i) => {
      it.index = i + 1
      it.status = it.index <= this.active ? 2 : i === this.active ? 1 : 0
      it.vertical = this.vertical
      if (i === 0) {
        it.setAttribute('first', '')
      } else if (i === this.children.length - 1) {
        it.setAttribute('last', '')
      }
    })
  }

  mounted() {
    this.$updateStepStatus()
  }
}

class Step extends Component {
  static props = {
    title: '',
    description: '',
    icon: { type: String, default: null },
    index: { type: Number, default: 1, attribute: false },
    status: { type: Number, default: 0, attribute: false },
    vertical: false
  }

  static styles = css`
    :host {
      flex: 1;
      display: flex;
      font-size: 14px;
    }
    .container {
      display: flex;
      flex-direction: column;
      align-items: center;
      width: 100%;
      color: var(--color-dark-1);
      user-select: none;

      &[status='2'] {
        color: var(--color-teal-1);

        .num {
          border-color: var(--color-teal-1);
        }
      }
      &[status='0'] {
        opacity: 0.6;
      }
    }

    .header {
      position: relative;
      display: flex;
      align-items: center;
      justify-content: center;
      width: 100%;
      height: 26px;

      &::before,
      &::after {
        position: absolute;
        left: 0;
        top: 12px;
        width: calc(50% - 24px);
        height: 1px;
        background: var(--color-grey-1);
        content: '';
      }

      &::after {
        left: unset;
        right: 0;
      }
    }

    .num {
      display: inline-flex;
      align-items: center;
      justify-content: center;
      width: 26px;
      height: 26px;
      border: 1px solid var(--color-grey-1);
      border-radius: 50%;
      --size: 14px;

      &.custom {
        border: 0;
        --size: 26px;
      }
    }
    .group {
      display: flex;
      flex-direction: column;
      align-items: center;
      justify-content: center;
    }
    .title {
      line-height: 2;
    }
    .description {
      line-height: 1.5;
      font-size: 12px;
    }

    :host([vertical]) {
      .container {
        flex-direction: row;
      }
      .header {
        width: 26px;
        min-height: 64px;

        &::before,
        &::after {
          top: 0;
          left: 12px;
          width: 1px;
          height: calc(50% - 16px);
        }
        &::after {
          top: unset;
          bottom: 0;
        }
      }
      .group {
        flex: 1;
        align-items: flex-start;
        margin-left: 12px;
      }
    }

    :host([first]) {
      .header:before {
        display: none;
      }
    }
    :host([last]) {
      .header:after {
        display: none;
      }
    }
  `

  render() {
    return html`
      <label class="container" status=${this.status}>
        <header class="header">
          <span class=${classMap({ num: true, custom: this.icon })}>
            ${this.icon
              ? html`<wc-icon name=${this.icon}></wc-icon>`
              : this.status === 2
              ? html`<wc-icon name="get"></wc-icon>`
              : this.index}
          </span>
        </header>
        <div class="group">
          <strong class="title">${this.title}</strong>
          <cite class="description">${this.description}</cite>
        </div>
      </label>
    `
  }
}

Steps.reg('steps')
Step.reg('step')
